var sys = require("sys");

var GenericClient = function(configuration, internalClient) {
  this.configuration = configuration;
  this.internalClient = internalClient;
  this.waitForCompletion = false;
  this.executingRequestNumber = 0;
}

GenericClient.prototype = {
    setErrorHandler: function(errorHandler) {
        console.log("setErrorHandler - errorHandler = "+errorHandler);
        this.errorHandler = errorHandler;
    },

    /*beginTransaction: function() {
        this.internalClient.query("BEGIN");
    },

    commit: function() {
        this.internalClient.query("COMMIT");
    },

    rollback: function() {
        this.internalClient.query("ROLLBACK");
    },

    createQuery: function(queryName, query) {
        return new PostgresqlQuery(this, queryName, query);
    },

    _convertDataAccessError: function(err) {
        return err;
    },*/

    _handleErrorIfNecessary: function(err) {
        ///TODO
        if (this.errorHandler!=null) {
            this.errorHandler(this._convertDataAccessError(err));
        } else {
            throw err;
        }
    },

    enableWaitForCompletion: function(onCompleteCallback) {
        this.waitForCompletion = true;
        this.onCompleteCallback = onCompleteCallback;
    },

    _triggerRequestStarted: function() {
        this.executingRequestNumber++;
    },

    _triggerRequestCompleted: function() {
        this.executingRequestNumber--;
        if (this.waitForCompletion && this.executingRequestNumber==0) {
            console.log("completed!!");
            console.log("this.internalClient.end()");
            this.internalClient.end();

            if (this.onCompleteCallback!=null) {
                this.onCompleteCallback(this);
            }
            this.waitForCompletion = false;
        }
    },

    /*getTableMetadatas: function(resultCallback) {
        this.executeQuery("SELECT tablename FROM pg_tables" +
                      " WHERE tablename NOT LIKE 'pg\\_%'" +
                      " AND tablename NOT LIKE 'sql\\_%'", resultCallback);
    },

    getColumnMetadatas: function(tableName, resultCallback) {
        this.executeQuery("SELECT attname FROM pg_attribute, pg_type" +
                      " WHERE typname = $1 AND attrelid = typrelid" +
                      " AND attname NOT IN ('cmin', 'cmax', 'ctid', 'oid', 'tableoid', 'xmin', 'xmax')",
                      tableName, resultCallback);
    },*/

    executeQuery: function() {
        var queryString = arguments[0];
        var parameters = arguments.length==3 ? arguments[1] : null;
        var resultCallback = arguments.length==3 ? arguments[2] : arguments[1];
        var self = this;
        if (parameters!=null) {
            console.log("1");
            self._triggerRequestStarted();
            self.internalClient.query({
                text: queryString,
                values: parameters
            }, function(err, result) {
                console.log("callback");
                console.log("err = "+err+", result = "+result);
                if (err!=null) {
                    self._handleErrorIfNecessary(err);
                } else {
                    resultCallback(result.rows);
                }
                self._triggerRequestCompleted();
            });
        } else {
            console.log("2");
            self._triggerRequestStarted();
            var query = self.internalClient.query(queryString, function(err, result) {
                console.log("callback");
                console.log("err = "+err+", result = "+result);
                if (err!=null) {
                    self._handleErrorIfNecessary(err);
                } else {
                    resultCallback(result.rows);
                }
                self._triggerRequestCompleted();
            });
        }
    },

    executeUpdate: function(query, parameters) {
        var self = this;
        if (parameters!=null) {
            console.log("executeUpdate1");
            self._triggerRequestStarted();
            var query = this.internalClient.query(query, parameters);
            query.on("error", function(err) {
                self._handleErrorIfNecessary(err);
                console.log("#### erreur : "+sys.inspect(err));
            });
            query.on("end", function() {
                self._triggerRequestCompleted();
            });
        } else {
            console.log("executeUpdate2");
            self._triggerRequestStarted();
            var query = this.internalClient.query(query);
            query.on("error", function(err) {
                self._handleErrorIfNecessary(err);
                console.log("#### erreur : "+sys.inspect(err));
            });
            query.on("end", function() {
                console.log("executeUpdate2 - end");
                self._triggerRequestCompleted();
            });
        }
    },

    end: function() {
        console.log("end");
        this.internalClient.end();
    },

    finalizePool: function() {
        console.log("configuration = "+sys.inspect(this.configuration));
        if (this.configuration.pooling) {
            var pg = this.configuration.native ? pgnative : pgjs;
            pg.end();
        }
    }
};

var PostgresqlQuery = function(client, queryName, query) {
    this.client = client;
    this.queryName = queryName;
    this.query = query;
}

PostgresqlQuery.prototype = {
    execute: function() {
        var parameters = null;
        var resultCallback = null;
        console.log("arguments.length = "+arguments.length);
        for (var i=0; i<arguments.length; i++) {
            console.log("- argument ("+i+"): "+arguments[i]);
        }
        if (arguments.length==2) {
            parameters = arguments[0];
            resultCallback = arguments[1];
        } else if (arguments.length==1) {
            if (typeof arguments[0] == "function") {
                resultCallback = arguments[0];
            } else if (arguments[0] instanceof Array) {
                parameters = arguments[0];
            }
        }
        console.log("parameters = "+parameters);
        console.log("resultCallback = "+resultCallback);

        var self = this;
        var queryConfig = {};
        queryConfig["name"] = this.queryName;
        queryConfig["text"] = this.query;
        if (parameters!=null) {
            queryConfig["values"] = parameters;
        }
        if (resultCallback!=null) {
            console.log("resultCallback!=null");
            self.client._triggerRequestStarted();
            this.client.internalQuery = this.client.internalClient.query(queryConfig, function(err, result) {
                self.client._handleErrorIfNecessary(err);
                if (!err) {
                    resultCallback(result.rows);
                }
                self.client._triggerRequestCompleted();
            });
        } else {
            console.log("resultCallback==null");
            self.client._triggerRequestStarted();
            this.internalQuery = this.client.internalClient.query(queryConfig);
            console.log("this.internalQuery = "+this.internalQuery);
            this.internalQuery.on("err", function() {
                self.client._handleErrorIfNecessary(err);
            });
            this.internalQuery.on("end", function() {
                self.client._triggerRequestCompleted();
            });
        }
    },
    listen: function(eventName, callback) {
        //suports event: result, row, end
        if (eventName=="result" || eventName=="row" || eventName=="end") {
            this.internalQuery.on(eventName, callback);
        }
    }
};

var PostgresqlDatabaseTemplate = function(configuration, callbacks) {
    this.configuration = configuration;
    this.callbacks = callbacks;
}

PostgresqlDatabaseTemplate.prototype = {
    _getOnCompleteCallback: function() {
        return (this.callbacks!=null && this.callbacks.onComplete
                 ? this.callbacks.onComplete : null);
    },

    _getErrorHandlerCallback: function() {
        return (this.callbacks!=null && this.callbacks.errorHandler
                 ? this.callbacks.errorHandler : null);
    },

    doOnData: function(callback) {
        var self = this;
        var pg = this.configuration.native ? pgnative : pgjs;
        console.log("native = "+this.configuration.native);
        console.log("pooling = "+this.configuration.pooling);
        if (this.configuration.pooling) {
            console.log("pooling");
            pg.connect({
                user: this.configuration.user,
                database: this.configuration.database,
                password: this.configuration.password,
                port: this.configuration.port,
                host: this.configuration.host
            }, function(err, internalClient) {
                console.log(sys.inspect(err));
                console.log("internalClient = "+internalClient);
                var client = new PostgresqlClient(self.configuration, internalClient);
                if (self._getErrorHandlerCallback()!=null) {
                    client.setErrorHandler(self._getErrorHandlerCallback());
                }
                callback(client);
                console.log("end of doOnData");
                client.enableWaitForCompletion(self._getOnCompleteCallback());
            });
        } else {
            var internalClient = new pg.Client({
                user: this.configuration.user,
                database: this.configuration.database,
                password: this.configuration.password,
                port: this.configuration.port,
                host: this.configuration.host
            });
            var client = new PostgresqlClient(this.configuration, internalClient);
            if (this._getErrorHandlerCallback()!=null) {
                client.setErrorHandler(this._getErrorHandlerCallback());
            }
            internalClient.connect();
            callback(client);
            client.enableWaitForCompletion(this._getOnCompleteCallback());
        }
    }
};

module.exports = {
    DatabaseTemplate: PostgresqlDatabaseTemplate
};
