var sys = require("sys");

var DriverTest = function(driver, config) {
    this.driver = driver;
    this.config = config;
}

DriverTest.prototype = {
    createDatabaseTemplate: function() {
        console.log(sys.inspect(this.driver));
        var callbacks = {};
        callbacks["errorHandler"] = function(err) {
            console.log("errorHandler = "+sys.inspect(err));
        };
        if (this.config["pooling"]) {
            callbacks["onComplete"] = function(client) {
                client.finalizePool();
            }
        }
        return new this.driver.DatabaseTemplate(this.config, callbacks);
    },

    executeTest: function() {
        var template = this.createDatabaseTemplate();
        template.doOnData(function(client) {
            client.executeQuery("select * from beatles", function(result) {
                console.log("in callback 1 - "+sys.inspect(result));
                for (var cpt=0;cpt<result.length;cpt++) {
                    console.log("- "+result[cpt].name);
                }
            });
            client.executeQuery("select * from beatles where name = $1", [ "Ringo" ], function(result) {
                console.log("in callback 2 - "+sys.inspect(result));
                for (var cpt=0;cpt<result.length;cpt++) {
                    console.log("- "+result[cpt].name);
                }
            });
            var query1 = client.createQuery("beatles1", "select * from beatles");
            query1.execute(function(result) {
                console.log("in callback 3 - "+sys.inspect(result));
                for (var cpt=0;cpt<result.length;cpt++) {
                    console.log("- "+result[cpt].name);
                }
            });
            var query2 = client.createQuery("beatles1", "select * from beatles");
            query2.execute();
            query2.listen("row", function(row) {
                console.log("in callback 4 - "+sys.inspect(row));
                console.log("- "+row.name);
            });
            console.log("########################");
            var query3 = client.createQuery("beatles2", "select * from beatles where name = $1");
            query3.execute([ "Ringo" ], function(result) {
                console.log("in callback 5 - "+sys.inspect(result));
                for (var cpt=0;cpt<result.length;cpt++) {
                    console.log("- "+result[cpt].name);
                }
            });
            console.log("########################");
            var query4 = client.createQuery("beatles2", "select * from beatles where name = $1");
            query4.execute([ "Ringo" ]);
            query4.listen("row", function(row) {
                console.log("in callback 6 - "+sys.inspect(row));
                console.log("- "+row.name);
            });
            client.getTableMetadata(function(result) {
                console.log("in callback 7 - "+sys.inspect(result));
            });
            client.getColumnMetadata("beatles", function(result) {
                console.log("in callback 8 - "+sys.inspect(result));
            });
        });
    }
}

module.exports = DriverTest;
